@@ -1,6 +1,7 @@
 module ActiveRecord
+  # = Active Record Belongs To Has One Association
   module Associations
-    class HasOneAssociation < BelongsToAssociation #:nodoc:
+    class HasOneAssociation < AssociationProxy #:nodoc:
       def initialize(owner, reflection)
         super
         construct_sql
@@ -8,18 +9,21 @@ module ActiveRecord
 
       def create(attrs = {}, replace_existing = true)
         new_record(replace_existing) do |reflection|
+          attrs = merge_with_conditions(attrs)
           reflection.create_association(attrs)
         end
       end
 
       def create!(attrs = {}, replace_existing = true)
         new_record(replace_existing) do |reflection|
+          attrs = merge_with_conditions(attrs)
           reflection.create_association!(attrs)
         end
       end
 
       def build(attrs = {}, replace_existing = true)
         new_record(replace_existing) do |reflection|
+          attrs = merge_with_conditions(attrs)
           reflection.build_association(attrs)
         end
       end
@@ -31,18 +35,18 @@ module ActiveRecord
           if dependent? && !dont_save
             case @reflection.options[:dependent]
             when :delete
-              @target.delete unless @target.new_record?
+              @target.delete if @target.persisted?
               @owner.clear_association_cache
             when :destroy
-              @target.destroy unless @target.new_record?
+              @target.destroy if @target.persisted?
               @owner.clear_association_cache
             when :nullify
               @target[@reflection.primary_key_name] = nil
-              @target.save unless @owner.new_record? || @target.new_record?
+              @target.save if @owner.persisted? && @target.persisted?
             end
           else
             @target[@reflection.primary_key_name] = nil
-            @target.save unless @owner.new_record? || @target.new_record?
+            @target.save if @owner.persisted? && @target.persisted?
           end
         end
 
@@ -54,9 +58,10 @@ module ActiveRecord
           @target = (AssociationProxy === obj ? obj.target : obj)
         end
 
+        set_inverse_instance(obj, @owner)
         @loaded = true
 
-        unless @owner.new_record? or obj.nil? or dont_save
+        unless !@owner.persisted? or obj.nil? or dont_save
           return (obj.save ? self : false)
         else
           return (obj.nil? ? nil : self)
@@ -74,19 +79,21 @@ module ActiveRecord
 
       private
         def find_target
-          @reflection.klass.find(:first, 
-            :conditions => @finder_sql,
-            :select     => @reflection.options[:select],
-            :order      => @reflection.options[:order], 
-            :include    => @reflection.options[:include],
-            :readonly   => @reflection.options[:readonly]
-          )
+          options = @reflection.options.dup
+          (options.keys - [:select, :order, :include, :readonly]).each do |key|
+            options.delete key
+          end
+          options[:conditions] = @finder_sql
+
+          the_target = @reflection.klass.find(:first, options)
+          set_inverse_instance(the_target, @owner)
+          the_target
         end
 
         def construct_sql
           case
             when @reflection.options[:as]
-              @finder_sql = 
+              @finder_sql =
                 "#{@reflection.quoted_table_name}.#{@reflection.options[:as]}_id = #{owner_quoted_id} AND " +
                 "#{@reflection.quoted_table_name}.#{@reflection.options[:as]}_type = #{@owner.class.quote_value(@owner.class.base_class.name.to_s)}"
             else
@@ -94,7 +101,7 @@ module ActiveRecord
           end
           @finder_sql << " AND (#{conditions})" if conditions
         end
-        
+
         def construct_scope
           create_scoping = {}
           set_belongs_to_association_for(create_scoping)
@@ -111,14 +118,26 @@ module ActiveRecord
           end
 
           if replace_existing
-            replace(record, true) 
+            replace(record, true)
           else
-            record[@reflection.primary_key_name] = @owner.id unless @owner.new_record?
+            record[@reflection.primary_key_name] = @owner.id if @owner.persisted?
             self.target = record
+            set_inverse_instance(record, @owner)
           end
 
           record
         end
+
+        def we_can_set_the_inverse_on_this?(record)
+          inverse = @reflection.inverse_of
+          return !inverse.nil?
+        end
+
+        def merge_with_conditions(attrs={})
+          attrs ||= {}
+          attrs.update(@reflection.options[:conditions]) if @reflection.options[:conditions].is_a?(Hash)
+          attrs
+        end
     end
   end
 end
