@@ -1,3 +1,5 @@
+require 'active_support/core_ext/big_decimal/conversions'
+
 module ActiveRecord
   module ConnectionAdapters # :nodoc:
     module Quoting
@@ -11,12 +13,12 @@ module ActiveRecord
           when String, ActiveSupport::Multibyte::Chars
             value = value.to_s
             if column && column.type == :binary && column.class.respond_to?(:string_to_binary)
-              "#{quoted_string_prefix}'#{quote_string(column.class.string_to_binary(value))}'" # ' (for ruby-mode)
+              "'#{quote_string(column.class.string_to_binary(value))}'" # ' (for ruby-mode)
             elsif column && [:integer, :float].include?(column.type)
               value = column.type == :integer ? value.to_i : value.to_f
               value.to_s
             else
-              "#{quoted_string_prefix}'#{quote_string(value)}'" # ' (for ruby-mode)
+              "'#{quote_string(value)}'" # ' (for ruby-mode)
             end
           when NilClass                 then "NULL"
           when TrueClass                then (column && column.type == :integer ? '1' : quoted_true)
@@ -28,7 +30,7 @@ module ActiveRecord
             if value.acts_like?(:date) || value.acts_like?(:time)
               "'#{quoted_date(value)}'"
             else
-              "#{quoted_string_prefix}'#{quote_string(value.to_yaml)}'"
+              "'#{quote_string(value.to_s)}'"
             end
         end
       end
@@ -58,11 +60,12 @@ module ActiveRecord
       end
 
       def quoted_date(value)
-        value.to_s(:db)
-      end
-
-      def quoted_string_prefix
-        ''
+        if value.acts_like?(:time)
+          zone_conversion_method = ActiveRecord::Base.default_timezone == :utc ? :getutc : :getlocal
+          value.respond_to?(zone_conversion_method) ? value.send(zone_conversion_method) : value
+        else
+          value
+        end.to_s(:db)
       end
     end
   end
